//******************************************************************************
//
// File:    Util.java
// Package: ---
// Unit:    Class Util
//
// This Java source file is copyright (C) 2017 by Alan Kaminsky. All rights
// reserved. For further information, contact the author, Alan Kaminsky, at
// ark@cs.rit.edu.
//
// This Java source file is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by the Free
// Software Foundation; either version 3 of the License, or (at your option) any
// later version.
//
// This Java source file is distributed in the hope that it will be useful, but
// WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
// FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for more
// details.
//
// A copy of the GNU General Public License is provided in the file gpl.txt. You
// may also obtain a copy of the GNU General Public License on the World Wide
// Web at http://www.gnu.org/licenses/gpl.html.
//
//******************************************************************************

import edu.rit.util.Random;
import edu.rit.util.RandomSubset;

/**
 * Class Util provides static utility methods for working with the LC4 cipher.
 *
 * @author  Alan Kaminsky
 * @version 28-Mar-2017
 */
public class Util
	{

// Prevent construction.

	private Util()
		{
		}

// Exported operations.

	/**
	 * Convert the given letter to an integer. Returns &minus;1 if illegal.
	 */
	public static int letterToInt
		(char c)
		{
		if (c == '#')
			return 0;
		else if (c == '_')
			return 1;
		else if ('2' <= c && c <= '9')
			return c - '2' + 2;
		else if ('A' <= c && c <= 'Z')
			return c - 'A' + 10;
		else if ('a' <= c && c <= 'z')
			return c - 'a' + 10;
		else
			return -1;
		}

	/**
	 * Convert the given integer to a letter. Returns <TT>'?'</TT> if illegal.
	 */
	public static char intToLetter
		(int x)
		{
		if (x == 0)
			return '#';
		else if (x == 1)
			return '_';
		else if (2 <= x && x <= 9)
			return (char)(x - 2 + '2');
		else if (10 <= x && x <= 35)
			return (char)(x - 10 + 'a');
		else
			return '?';
		}

	/**
	 * Convert the given string to an integer array.
	 */
	public static int[] lettersToInts
		(String s)
		{
		int[] x = new int [s.length()];
		for (int i = 0; i < x.length; ++ i)
			x[i] = letterToInt (s.charAt (i));
		return x;
		}

	/**
	 * Convert the given integer array to a string.
	 */
	public static String intsToLetters
		(int[] x)
		{
		StringBuilder b = new StringBuilder();
		for (int i = 0; i < x.length; ++ i)
			b.append (intToLetter (x[i]));
		return b.toString();
		}

	/**
	 * Generate a random LC4 key.
	 */
	public static void genKey
		(int[] key,
		 RandomSubset rs)
		{
		rs.restart();
		for (int i = 0; i < 36; ++ i)
			key[i] = rs.next();
		}

	/**
	 * Generate a random LC4 nonce.
	 */
	public static void genNonce
		(int[] nonce,
		 Random prng)
		{
		for (int i = 0; i < nonce.length; ++ i)
			nonce[i] = prng.nextInt (36);
		}

	/**
	 * Generate a random plaintext using the given digram frequencies.
	 */
	public static void genPlaintext
		(int[] pt,
		 DigramPicker dp)
		{
		dp.start();
		for (int i = 0; i < pt.length; ++ i)
			pt[i] = dp.nextInt() + 10;
		}

	}
